# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Update(AAZCommand):
    """Update for Creates or updates an App Service Plan.
    """

    _aaz_info = {
        "version": "2025-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.web/serverfarms/{}", "2025-03-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Name of the App Service plan.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "AppServicePlan"

        _args_schema = cls._args_schema
        _args_schema.extended_location = AAZObjectArg(
            options=["--extended-location"],
            arg_group="AppServicePlan",
            help="Extended Location.",
            nullable=True,
        )
        _args_schema.kind = AAZStrArg(
            options=["--kind"],
            arg_group="AppServicePlan",
            help="Kind of resource. If the resource is an app, you can refer to https://github.com/Azure/app-service-linux-docs/blob/master/Things_You_Should_Know/kind_property.md#app-service-resource-kind-reference for details supported values for kind.",
            nullable=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            arg_group="AppServicePlan",
            help="Resource Location.",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.sku = AAZObjectArg(
            options=["--sku"],
            arg_group="AppServicePlan",
            help="Description of a SKU for a scalable resource.",
            nullable=True,
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="AppServicePlan",
            help="Resource tags.",
            nullable=True,
        )

        extended_location = cls._args_schema.extended_location
        extended_location.name = AAZStrArg(
            options=["name"],
            help="Name of extended location.",
            nullable=True,
        )

        sku = cls._args_schema.sku
        sku.capabilities = AAZListArg(
            options=["capabilities"],
            help="Capabilities of the SKU, e.g., is traffic manager enabled?",
            nullable=True,
        )
        sku.capacity = AAZIntArg(
            options=["capacity"],
            help="Current number of instances assigned to the resource.",
            nullable=True,
        )
        sku.family = AAZStrArg(
            options=["family"],
            help="Family code of the resource SKU.",
            nullable=True,
        )
        sku.locations = AAZListArg(
            options=["locations"],
            help="Locations of the SKU.",
            nullable=True,
        )
        sku.name = AAZStrArg(
            options=["name"],
            help="Name of the resource SKU.",
            nullable=True,
        )
        sku.size = AAZStrArg(
            options=["size"],
            help="Size specifier of the resource SKU.",
            nullable=True,
        )
        sku.sku_capacity = AAZObjectArg(
            options=["sku-capacity"],
            help="Min, max, and default scale values of the SKU.",
            nullable=True,
        )
        sku.tier = AAZStrArg(
            options=["tier"],
            help="Service tier of the resource SKU.",
            nullable=True,
        )

        capabilities = cls._args_schema.sku.capabilities
        capabilities.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.sku.capabilities.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="Name of the SKU capability.",
            nullable=True,
        )
        _element.reason = AAZStrArg(
            options=["reason"],
            help="Reason of the SKU capability.",
            nullable=True,
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="Value of the SKU capability.",
            nullable=True,
        )

        locations = cls._args_schema.sku.locations
        locations.Element = AAZStrArg(
            nullable=True,
        )

        sku_capacity = cls._args_schema.sku.sku_capacity
        sku_capacity.default = AAZIntArg(
            options=["default"],
            help="Default number of workers for this App Service plan SKU.",
            nullable=True,
        )
        sku_capacity.elastic_maximum = AAZIntArg(
            options=["elastic-maximum"],
            help="Maximum number of Elastic workers for this App Service plan SKU.",
            nullable=True,
        )
        sku_capacity.maximum = AAZIntArg(
            options=["maximum"],
            help="Maximum number of workers for this App Service plan SKU.",
            nullable=True,
        )
        sku_capacity.minimum = AAZIntArg(
            options=["minimum"],
            help="Minimum number of workers for this App Service plan SKU.",
            nullable=True,
        )
        sku_capacity.scale_type = AAZStrArg(
            options=["scale-type"],
            help="Available scale configurations for an App Service plan.",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Identity"

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.async_scaling_enabled = AAZBoolArg(
            options=["--async-scaling-enabled"],
            arg_group="Properties",
            help="If <code>true</code>, this App Service Plan will attempt to scale asynchronously if there are insufficient workers to scale synchronously. If <code>false</code>, this App Service Plan will only attempt sync scaling.",
            nullable=True,
        )
        _args_schema.elastic_scale_enabled = AAZBoolArg(
            options=["--elastic-scale-enabled"],
            arg_group="Properties",
            help="ServerFarm supports ElasticScale. Apps in this plan will scale as if the ServerFarm was ElasticPremium sku",
            nullable=True,
        )
        _args_schema.free_offer_expiration_time = AAZDateTimeArg(
            options=["--free-offer-expiration-time"],
            arg_group="Properties",
            help="The time when the server farm free offer expires.",
            nullable=True,
            fmt=AAZDateTimeFormat(
                protocol="iso",
            ),
        )
        _args_schema.install_scripts = AAZListArg(
            options=["--install-scripts"],
            arg_group="Properties",
            help="Install scripts associated with this App Service plan.",
            nullable=True,
        )
        _args_schema.is_custom_mode = AAZBoolArg(
            options=["--is-custom-mode"],
            arg_group="Properties",
            help="Whether this server farm is in custom mode.",
            nullable=True,
        )
        _args_schema.is_spot = AAZBoolArg(
            options=["--is-spot"],
            arg_group="Properties",
            help="If <code>true</code>, this App Service Plan owns spot instances.",
            nullable=True,
        )
        _args_schema.kube_environment_profile = AAZObjectArg(
            options=["--kube-environment-profile"],
            arg_group="Properties",
            help="Specification for the Kubernetes Environment to use for the App Service plan.",
            nullable=True,
        )
        _args_schema.maximum_elastic_worker_count = AAZIntArg(
            options=["--maximum-elastic-worker-count"],
            arg_group="Properties",
            help="Maximum number of total workers allowed for this ElasticScaleEnabled App Service Plan",
            nullable=True,
        )
        _args_schema.network = AAZObjectArg(
            options=["--network"],
            arg_group="Properties",
            help="All network settings for the server farm.",
            nullable=True,
        )
        _args_schema.per_site_scaling = AAZBoolArg(
            options=["--per-site-scaling"],
            arg_group="Properties",
            help="If <code>true</code>, apps assigned to this App Service plan can be scaled independently. If <code>false</code>, apps assigned to this App Service plan will scale to all instances of the plan.",
            nullable=True,
        )
        _args_schema.plan_default_identity = AAZObjectArg(
            options=["--plan-default-identity"],
            arg_group="Properties",
            help="Identity to use by platform for various features and integrations using managed identity.",
            nullable=True,
        )
        _args_schema.rdp_enabled = AAZBoolArg(
            options=["--rdp-enabled"],
            arg_group="Properties",
            help="If <code>true</code>, RDP access is enabled for this App Service plan. Only applicable for IsCustomMode ASPs. If <code>false</code>, RDP access is disabled.",
            nullable=True,
        )
        _args_schema.registry_adapters = AAZListArg(
            options=["--registry-adapters"],
            arg_group="Properties",
            help="Registry adapters associated with this App Service plan.",
            nullable=True,
        )
        _args_schema.spot_expiration_time = AAZDateTimeArg(
            options=["--spot-expiration-time"],
            arg_group="Properties",
            help="The time when the server farm expires. Valid only if it is a spot server farm.",
            nullable=True,
            fmt=AAZDateTimeFormat(
                protocol="iso",
            ),
        )
        _args_schema.storage_mounts = AAZListArg(
            options=["--storage-mounts"],
            arg_group="Properties",
            help="Storage mounts associated with this App Service plan.",
            nullable=True,
        )
        _args_schema.target_worker_count = AAZIntArg(
            options=["--target-worker-count"],
            arg_group="Properties",
            help="Scaling worker count.",
            nullable=True,
        )
        _args_schema.target_worker_size_id = AAZIntArg(
            options=["--target-worker-size-id"],
            arg_group="Properties",
            help="Scaling worker size ID.",
            nullable=True,
        )
        _args_schema.worker_tier_name = AAZStrArg(
            options=["--worker-tier-name"],
            arg_group="Properties",
            help="Target worker tier assigned to the App Service plan.",
            nullable=True,
        )
        _args_schema.zone_redundant = AAZBoolArg(
            options=["--zone-redundant"],
            arg_group="Properties",
            help="If <code>true</code>, this App Service Plan will perform availability zone balancing. If <code>false</code>, this App Service Plan will not perform availability zone balancing.",
            nullable=True,
        )

        install_scripts = cls._args_schema.install_scripts
        install_scripts.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.install_scripts.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="Name of the install script.",
            nullable=True,
        )
        _element.source = AAZObjectArg(
            options=["source"],
            help="Source of the install script.",
            nullable=True,
        )

        source = cls._args_schema.install_scripts.Element.source
        source.source_uri = AAZStrArg(
            options=["source-uri"],
            help="Install script source URI where the install script file will be fetched from.",
            nullable=True,
        )
        source.type = AAZStrArg(
            options=["type"],
            help="Type of the install script.",
            nullable=True,
            enum={"PlatformStorage": "PlatformStorage", "RemoteAzureBlob": "RemoteAzureBlob"},
        )

        kube_environment_profile = cls._args_schema.kube_environment_profile
        kube_environment_profile.id = AAZStrArg(
            options=["id"],
            help="Resource ID of the Kubernetes Environment.",
            nullable=True,
        )

        network = cls._args_schema.network
        network.virtual_network_subnet_id = AAZStrArg(
            options=["virtual-network-subnet-id"],
            help="Azure Resource Manager ID of the Virtual network and subnet to be joined by Regional VNET Integration. This must be of the form /subscriptions/{subscriptionName}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/virtualNetworks/{vnetName}/subnets/{subnetName}",
            nullable=True,
        )

        plan_default_identity = cls._args_schema.plan_default_identity
        plan_default_identity.identity_type = AAZStrArg(
            options=["identity-type"],
            nullable=True,
            enum={"None": "None", "SystemAssigned": "SystemAssigned", "UserAssigned": "UserAssigned"},
        )
        plan_default_identity.user_assigned_identity_resource_id = AAZStrArg(
            options=["user-assigned-identity-resource-id"],
            nullable=True,
        )

        registry_adapters = cls._args_schema.registry_adapters
        registry_adapters.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.registry_adapters.Element
        _element.key_vault_secret_reference = AAZObjectArg(
            options=["key-vault-secret-reference"],
            help="Key vault reference to the value that will be placed in the registry location",
            nullable=True,
        )
        cls._build_args_key_vault_reference_with_status_update(_element.key_vault_secret_reference)
        _element.registry_key = AAZStrArg(
            options=["registry-key"],
            help="Registry key for the adapter.",
            nullable=True,
        )
        _element.type = AAZStrArg(
            options=["type"],
            help="Type of the registry adapter.",
            nullable=True,
            enum={"Binary": "Binary", "DWord": "DWord", "Expand_String": "Expand_String", "Multi_String": "Multi_String", "QWord": "QWord", "String": "String"},
        )

        storage_mounts = cls._args_schema.storage_mounts
        storage_mounts.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.storage_mounts.Element
        _element.credentials_key_vault_reference = AAZObjectArg(
            options=["credentials-key-vault-reference"],
            help="KV reference to the credentials to connect to the share.",
            nullable=True,
        )
        cls._build_args_key_vault_reference_with_status_update(_element.credentials_key_vault_reference)
        _element.destination_path = AAZStrArg(
            options=["destination-path"],
            help="Path on worker where storage will be mounted.",
            nullable=True,
        )
        _element.name = AAZStrArg(
            options=["name"],
            help="Name of the storage mount.",
            nullable=True,
        )
        _element.source = AAZStrArg(
            options=["source"],
            help="Source of the fileshare/storage.",
            nullable=True,
        )
        _element.type = AAZStrArg(
            options=["type"],
            help="Type of the storage mount.",
            nullable=True,
            enum={"AzureFiles": "AzureFiles", "FileShare": "FileShare", "LocalStorage": "LocalStorage"},
        )
        return cls._args_schema

    _args_key_vault_reference_with_status_update = None

    @classmethod
    def _build_args_key_vault_reference_with_status_update(cls, _schema):
        if cls._args_key_vault_reference_with_status_update is not None:
            _schema.reference_status = cls._args_key_vault_reference_with_status_update.reference_status
            _schema.secret_uri = cls._args_key_vault_reference_with_status_update.secret_uri
            return

        cls._args_key_vault_reference_with_status_update = AAZObjectArg(
            nullable=True,
        )

        key_vault_reference_with_status_update = cls._args_key_vault_reference_with_status_update
        key_vault_reference_with_status_update.reference_status = AAZStrArg(
            options=["reference-status"],
            help="Reference status of the key vault secret.",
            nullable=True,
        )
        key_vault_reference_with_status_update.secret_uri = AAZStrArg(
            options=["secret-uri"],
            help="Key vault secret URI.",
            nullable=True,
        )

        _schema.reference_status = cls._args_key_vault_reference_with_status_update.reference_status
        _schema.secret_uri = cls._args_key_vault_reference_with_status_update.secret_uri

    def _execute_operations(self):
        self.pre_operations()
        self.AppServicePlansGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.AppServicePlansCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AppServicePlansGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/serverfarms/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_app_service_plan_read(cls._schema_on_200)

            return cls._schema_on_200

    class AppServicePlansCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/serverfarms/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_app_service_plan_read(cls._schema_on_200)

            return cls._schema_on_200

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("extendedLocation", AAZObjectType, ".extended_location")
            _builder.set_prop("identity", AAZIdentityObjectType)
            _builder.set_prop("kind", AAZStrType, ".kind")
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("sku", AAZObjectType, ".sku")
            _builder.set_prop("tags", AAZDictType, ".tags")

            extended_location = _builder.get(".extendedLocation")
            if extended_location is not None:
                extended_location.set_prop("name", AAZStrType, ".name")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("asyncScalingEnabled", AAZBoolType, ".async_scaling_enabled")
                properties.set_prop("elasticScaleEnabled", AAZBoolType, ".elastic_scale_enabled")
                properties.set_prop("freeOfferExpirationTime", AAZStrType, ".free_offer_expiration_time")
                properties.set_prop("installScripts", AAZListType, ".install_scripts")
                properties.set_prop("isCustomMode", AAZBoolType, ".is_custom_mode")
                properties.set_prop("isSpot", AAZBoolType, ".is_spot")
                properties.set_prop("kubeEnvironmentProfile", AAZObjectType, ".kube_environment_profile")
                properties.set_prop("maximumElasticWorkerCount", AAZIntType, ".maximum_elastic_worker_count")
                properties.set_prop("network", AAZObjectType, ".network")
                properties.set_prop("perSiteScaling", AAZBoolType, ".per_site_scaling")
                properties.set_prop("planDefaultIdentity", AAZObjectType, ".plan_default_identity")
                properties.set_prop("rdpEnabled", AAZBoolType, ".rdp_enabled")
                properties.set_prop("registryAdapters", AAZListType, ".registry_adapters")
                properties.set_prop("spotExpirationTime", AAZStrType, ".spot_expiration_time")
                properties.set_prop("storageMounts", AAZListType, ".storage_mounts")
                properties.set_prop("targetWorkerCount", AAZIntType, ".target_worker_count")
                properties.set_prop("targetWorkerSizeId", AAZIntType, ".target_worker_size_id")
                properties.set_prop("workerTierName", AAZStrType, ".worker_tier_name")
                properties.set_prop("zoneRedundant", AAZBoolType, ".zone_redundant")

            install_scripts = _builder.get(".properties.installScripts")
            if install_scripts is not None:
                install_scripts.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.installScripts[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("source", AAZObjectType, ".source")

            source = _builder.get(".properties.installScripts[].source")
            if source is not None:
                source.set_prop("sourceUri", AAZStrType, ".source_uri")
                source.set_prop("type", AAZStrType, ".type")

            kube_environment_profile = _builder.get(".properties.kubeEnvironmentProfile")
            if kube_environment_profile is not None:
                kube_environment_profile.set_prop("id", AAZStrType, ".id")

            network = _builder.get(".properties.network")
            if network is not None:
                network.set_prop("virtualNetworkSubnetId", AAZStrType, ".virtual_network_subnet_id")

            plan_default_identity = _builder.get(".properties.planDefaultIdentity")
            if plan_default_identity is not None:
                plan_default_identity.set_prop("identityType", AAZStrType, ".identity_type")
                plan_default_identity.set_prop("userAssignedIdentityResourceId", AAZStrType, ".user_assigned_identity_resource_id")

            registry_adapters = _builder.get(".properties.registryAdapters")
            if registry_adapters is not None:
                registry_adapters.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.registryAdapters[]")
            if _elements is not None:
                _UpdateHelper._build_schema_key_vault_reference_with_status_update(_elements.set_prop("keyVaultSecretReference", AAZObjectType, ".key_vault_secret_reference"))
                _elements.set_prop("registryKey", AAZStrType, ".registry_key")
                _elements.set_prop("type", AAZStrType, ".type")

            storage_mounts = _builder.get(".properties.storageMounts")
            if storage_mounts is not None:
                storage_mounts.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.storageMounts[]")
            if _elements is not None:
                _UpdateHelper._build_schema_key_vault_reference_with_status_update(_elements.set_prop("credentialsKeyVaultReference", AAZObjectType, ".credentials_key_vault_reference"))
                _elements.set_prop("destinationPath", AAZStrType, ".destination_path")
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("source", AAZStrType, ".source")
                _elements.set_prop("type", AAZStrType, ".type")

            sku = _builder.get(".sku")
            if sku is not None:
                sku.set_prop("capabilities", AAZListType, ".capabilities")
                sku.set_prop("capacity", AAZIntType, ".capacity")
                sku.set_prop("family", AAZStrType, ".family")
                sku.set_prop("locations", AAZListType, ".locations")
                sku.set_prop("name", AAZStrType, ".name")
                sku.set_prop("size", AAZStrType, ".size")
                sku.set_prop("skuCapacity", AAZObjectType, ".sku_capacity")
                sku.set_prop("tier", AAZStrType, ".tier")

            capabilities = _builder.get(".sku.capabilities")
            if capabilities is not None:
                capabilities.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".sku.capabilities[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("reason", AAZStrType, ".reason")
                _elements.set_prop("value", AAZStrType, ".value")

            locations = _builder.get(".sku.locations")
            if locations is not None:
                locations.set_elements(AAZStrType, ".")

            sku_capacity = _builder.get(".sku.skuCapacity")
            if sku_capacity is not None:
                sku_capacity.set_prop("default", AAZIntType, ".default")
                sku_capacity.set_prop("elasticMaximum", AAZIntType, ".elastic_maximum")
                sku_capacity.set_prop("maximum", AAZIntType, ".maximum")
                sku_capacity.set_prop("minimum", AAZIntType, ".minimum")
                sku_capacity.set_prop("scaleType", AAZStrType, ".scale_type")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_key_vault_reference_with_status_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("referenceStatus", AAZStrType, ".reference_status")
        _builder.set_prop("secretUri", AAZStrType, ".secret_uri")

    _schema_app_service_plan_read = None

    @classmethod
    def _build_schema_app_service_plan_read(cls, _schema):
        if cls._schema_app_service_plan_read is not None:
            _schema.extended_location = cls._schema_app_service_plan_read.extended_location
            _schema.id = cls._schema_app_service_plan_read.id
            _schema.identity = cls._schema_app_service_plan_read.identity
            _schema.kind = cls._schema_app_service_plan_read.kind
            _schema.location = cls._schema_app_service_plan_read.location
            _schema.name = cls._schema_app_service_plan_read.name
            _schema.properties = cls._schema_app_service_plan_read.properties
            _schema.sku = cls._schema_app_service_plan_read.sku
            _schema.tags = cls._schema_app_service_plan_read.tags
            _schema.type = cls._schema_app_service_plan_read.type
            return

        cls._schema_app_service_plan_read = _schema_app_service_plan_read = AAZObjectType()

        app_service_plan_read = _schema_app_service_plan_read
        app_service_plan_read.extended_location = AAZObjectType(
            serialized_name="extendedLocation",
        )
        app_service_plan_read.id = AAZStrType(
            flags={"read_only": True},
        )
        app_service_plan_read.identity = AAZIdentityObjectType()
        app_service_plan_read.kind = AAZStrType()
        app_service_plan_read.location = AAZStrType(
            flags={"required": True},
        )
        app_service_plan_read.name = AAZStrType(
            flags={"read_only": True},
        )
        app_service_plan_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        app_service_plan_read.sku = AAZObjectType()
        app_service_plan_read.tags = AAZDictType()
        app_service_plan_read.type = AAZStrType(
            flags={"read_only": True},
        )

        extended_location = _schema_app_service_plan_read.extended_location
        extended_location.name = AAZStrType()
        extended_location.type = AAZStrType(
            flags={"read_only": True},
        )

        identity = _schema_app_service_plan_read.identity
        identity.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )
        identity.tenant_id = AAZStrType(
            serialized_name="tenantId",
            flags={"read_only": True},
        )
        identity.type = AAZStrType()
        identity.user_assigned_identities = AAZDictType(
            serialized_name="userAssignedIdentities",
        )

        user_assigned_identities = _schema_app_service_plan_read.identity.user_assigned_identities
        user_assigned_identities.Element = AAZObjectType()

        _element = _schema_app_service_plan_read.identity.user_assigned_identities.Element
        _element.client_id = AAZStrType(
            serialized_name="clientId",
            flags={"read_only": True},
        )
        _element.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )

        properties = _schema_app_service_plan_read.properties
        properties.async_scaling_enabled = AAZBoolType(
            serialized_name="asyncScalingEnabled",
        )
        properties.elastic_scale_enabled = AAZBoolType(
            serialized_name="elasticScaleEnabled",
        )
        properties.free_offer_expiration_time = AAZStrType(
            serialized_name="freeOfferExpirationTime",
        )
        properties.geo_region = AAZStrType(
            serialized_name="geoRegion",
            flags={"read_only": True},
        )
        properties.hosting_environment_profile = AAZObjectType(
            serialized_name="hostingEnvironmentProfile",
        )
        properties.hyper_v = AAZBoolType(
            serialized_name="hyperV",
        )
        properties.install_scripts = AAZListType(
            serialized_name="installScripts",
        )
        properties.is_custom_mode = AAZBoolType(
            serialized_name="isCustomMode",
        )
        properties.is_spot = AAZBoolType(
            serialized_name="isSpot",
        )
        properties.is_xenon = AAZBoolType(
            serialized_name="isXenon",
        )
        properties.kube_environment_profile = AAZObjectType(
            serialized_name="kubeEnvironmentProfile",
        )
        properties.maximum_elastic_worker_count = AAZIntType(
            serialized_name="maximumElasticWorkerCount",
        )
        properties.maximum_number_of_workers = AAZIntType(
            serialized_name="maximumNumberOfWorkers",
            flags={"read_only": True},
        )
        properties.network = AAZObjectType()
        properties.number_of_sites = AAZIntType(
            serialized_name="numberOfSites",
            flags={"read_only": True},
        )
        properties.number_of_workers = AAZIntType(
            serialized_name="numberOfWorkers",
            flags={"read_only": True},
        )
        properties.per_site_scaling = AAZBoolType(
            serialized_name="perSiteScaling",
        )
        properties.plan_default_identity = AAZObjectType(
            serialized_name="planDefaultIdentity",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.rdp_enabled = AAZBoolType(
            serialized_name="rdpEnabled",
        )
        properties.registry_adapters = AAZListType(
            serialized_name="registryAdapters",
        )
        properties.reserved = AAZBoolType()
        properties.resource_group = AAZStrType(
            serialized_name="resourceGroup",
            flags={"read_only": True},
        )
        properties.spot_expiration_time = AAZStrType(
            serialized_name="spotExpirationTime",
        )
        properties.status = AAZStrType(
            flags={"read_only": True},
        )
        properties.storage_mounts = AAZListType(
            serialized_name="storageMounts",
        )
        properties.subscription = AAZStrType(
            flags={"read_only": True},
        )
        properties.target_worker_count = AAZIntType(
            serialized_name="targetWorkerCount",
        )
        properties.target_worker_size_id = AAZIntType(
            serialized_name="targetWorkerSizeId",
        )
        properties.worker_tier_name = AAZStrType(
            serialized_name="workerTierName",
        )
        properties.zone_redundant = AAZBoolType(
            serialized_name="zoneRedundant",
        )

        hosting_environment_profile = _schema_app_service_plan_read.properties.hosting_environment_profile
        hosting_environment_profile.id = AAZStrType()
        hosting_environment_profile.name = AAZStrType(
            flags={"read_only": True},
        )
        hosting_environment_profile.type = AAZStrType(
            flags={"read_only": True},
        )

        install_scripts = _schema_app_service_plan_read.properties.install_scripts
        install_scripts.Element = AAZObjectType()

        _element = _schema_app_service_plan_read.properties.install_scripts.Element
        _element.name = AAZStrType()
        _element.source = AAZObjectType()

        source = _schema_app_service_plan_read.properties.install_scripts.Element.source
        source.source_uri = AAZStrType(
            serialized_name="sourceUri",
        )
        source.type = AAZStrType()

        kube_environment_profile = _schema_app_service_plan_read.properties.kube_environment_profile
        kube_environment_profile.id = AAZStrType()
        kube_environment_profile.name = AAZStrType(
            flags={"read_only": True},
        )
        kube_environment_profile.type = AAZStrType(
            flags={"read_only": True},
        )

        network = _schema_app_service_plan_read.properties.network
        network.virtual_network_subnet_id = AAZStrType(
            serialized_name="virtualNetworkSubnetId",
        )

        plan_default_identity = _schema_app_service_plan_read.properties.plan_default_identity
        plan_default_identity.identity_type = AAZStrType(
            serialized_name="identityType",
        )
        plan_default_identity.user_assigned_identity_resource_id = AAZStrType(
            serialized_name="userAssignedIdentityResourceId",
        )

        registry_adapters = _schema_app_service_plan_read.properties.registry_adapters
        registry_adapters.Element = AAZObjectType()

        _element = _schema_app_service_plan_read.properties.registry_adapters.Element
        _element.key_vault_secret_reference = AAZObjectType(
            serialized_name="keyVaultSecretReference",
        )
        cls._build_schema_key_vault_reference_with_status_read(_element.key_vault_secret_reference)
        _element.registry_key = AAZStrType(
            serialized_name="registryKey",
        )
        _element.type = AAZStrType()

        storage_mounts = _schema_app_service_plan_read.properties.storage_mounts
        storage_mounts.Element = AAZObjectType()

        _element = _schema_app_service_plan_read.properties.storage_mounts.Element
        _element.credentials_key_vault_reference = AAZObjectType(
            serialized_name="credentialsKeyVaultReference",
        )
        cls._build_schema_key_vault_reference_with_status_read(_element.credentials_key_vault_reference)
        _element.destination_path = AAZStrType(
            serialized_name="destinationPath",
        )
        _element.name = AAZStrType()
        _element.source = AAZStrType()
        _element.type = AAZStrType()

        sku = _schema_app_service_plan_read.sku
        sku.capabilities = AAZListType()
        sku.capacity = AAZIntType()
        sku.family = AAZStrType()
        sku.locations = AAZListType()
        sku.name = AAZStrType()
        sku.size = AAZStrType()
        sku.sku_capacity = AAZObjectType(
            serialized_name="skuCapacity",
        )
        sku.tier = AAZStrType()

        capabilities = _schema_app_service_plan_read.sku.capabilities
        capabilities.Element = AAZObjectType()

        _element = _schema_app_service_plan_read.sku.capabilities.Element
        _element.name = AAZStrType()
        _element.reason = AAZStrType()
        _element.value = AAZStrType()

        locations = _schema_app_service_plan_read.sku.locations
        locations.Element = AAZStrType()

        sku_capacity = _schema_app_service_plan_read.sku.sku_capacity
        sku_capacity.default = AAZIntType()
        sku_capacity.elastic_maximum = AAZIntType(
            serialized_name="elasticMaximum",
        )
        sku_capacity.maximum = AAZIntType()
        sku_capacity.minimum = AAZIntType()
        sku_capacity.scale_type = AAZStrType(
            serialized_name="scaleType",
        )

        tags = _schema_app_service_plan_read.tags
        tags.Element = AAZStrType()

        _schema.extended_location = cls._schema_app_service_plan_read.extended_location
        _schema.id = cls._schema_app_service_plan_read.id
        _schema.identity = cls._schema_app_service_plan_read.identity
        _schema.kind = cls._schema_app_service_plan_read.kind
        _schema.location = cls._schema_app_service_plan_read.location
        _schema.name = cls._schema_app_service_plan_read.name
        _schema.properties = cls._schema_app_service_plan_read.properties
        _schema.sku = cls._schema_app_service_plan_read.sku
        _schema.tags = cls._schema_app_service_plan_read.tags
        _schema.type = cls._schema_app_service_plan_read.type

    _schema_key_vault_reference_with_status_read = None

    @classmethod
    def _build_schema_key_vault_reference_with_status_read(cls, _schema):
        if cls._schema_key_vault_reference_with_status_read is not None:
            _schema.reference_status = cls._schema_key_vault_reference_with_status_read.reference_status
            _schema.secret_uri = cls._schema_key_vault_reference_with_status_read.secret_uri
            return

        cls._schema_key_vault_reference_with_status_read = _schema_key_vault_reference_with_status_read = AAZObjectType()

        key_vault_reference_with_status_read = _schema_key_vault_reference_with_status_read
        key_vault_reference_with_status_read.reference_status = AAZStrType(
            serialized_name="referenceStatus",
        )
        key_vault_reference_with_status_read.secret_uri = AAZStrType(
            serialized_name="secretUri",
        )

        _schema.reference_status = cls._schema_key_vault_reference_with_status_read.reference_status
        _schema.secret_uri = cls._schema_key_vault_reference_with_status_read.secret_uri


__all__ = ["Update"]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AppInsightsReference
    from ._models_py3 import AutoScaleSettings
    from ._models_py3 import AzureBlobFileSystemReference
    from ._models_py3 import AzureFileShareReference
    from ._models_py3 import AzureStorageCredentialsInfo
    from ._models_py3 import BatchAIError
    from ._models_py3 import CNTKsettings
    from ._models_py3 import Caffe2Settings
    from ._models_py3 import CaffeSettings
    from ._models_py3 import ChainerSettings
    from ._models_py3 import CloudErrorBody
    from ._models_py3 import Cluster
    from ._models_py3 import ClusterCreateParameters
    from ._models_py3 import ClusterListResult
    from ._models_py3 import ClusterUpdateParameters
    from ._models_py3 import ClustersListByWorkspaceOptions
    from ._models_py3 import ContainerSettings
    from ._models_py3 import CustomMpiSettings
    from ._models_py3 import CustomToolkitSettings
    from ._models_py3 import DataDisks
    from ._models_py3 import EnvironmentVariable
    from ._models_py3 import EnvironmentVariableWithSecretValue
    from ._models_py3 import Experiment
    from ._models_py3 import ExperimentListResult
    from ._models_py3 import ExperimentsListByWorkspaceOptions
    from ._models_py3 import File
    from ._models_py3 import FileListResult
    from ._models_py3 import FileServer
    from ._models_py3 import FileServerCreateParameters
    from ._models_py3 import FileServerListResult
    from ._models_py3 import FileServerReference
    from ._models_py3 import FileServersListByWorkspaceOptions
    from ._models_py3 import HorovodSettings
    from ._models_py3 import ImageReference
    from ._models_py3 import ImageSourceRegistry
    from ._models_py3 import InputDirectory
    from ._models_py3 import Job
    from ._models_py3 import JobBasePropertiesConstraints
    from ._models_py3 import JobCreateParameters
    from ._models_py3 import JobListResult
    from ._models_py3 import JobPreparation
    from ._models_py3 import JobPropertiesConstraints
    from ._models_py3 import JobPropertiesExecutionInfo
    from ._models_py3 import JobsListByExperimentOptions
    from ._models_py3 import JobsListOutputFilesOptions
    from ._models_py3 import KeyVaultSecretReference
    from ._models_py3 import ListUsagesResult
    from ._models_py3 import ManualScaleSettings
    from ._models_py3 import MountSettings
    from ._models_py3 import MountVolumes
    from ._models_py3 import NameValuePair
    from ._models_py3 import NodeSetup
    from ._models_py3 import NodeStateCounts
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import OutputDirectory
    from ._models_py3 import PerformanceCountersSettings
    from ._models_py3 import PrivateRegistryCredentials
    from ._models_py3 import ProxyResource
    from ._models_py3 import PyTorchSettings
    from ._models_py3 import RemoteLoginInformation
    from ._models_py3 import RemoteLoginInformationListResult
    from ._models_py3 import Resource
    from ._models_py3 import ResourceId
    from ._models_py3 import ScaleSettings
    from ._models_py3 import SetupTask
    from ._models_py3 import SshConfiguration
    from ._models_py3 import TensorFlowSettings
    from ._models_py3 import UnmanagedFileSystemReference
    from ._models_py3 import Usage
    from ._models_py3 import UsageName
    from ._models_py3 import UserAccountSettings
    from ._models_py3 import VirtualMachineConfiguration
    from ._models_py3 import Workspace
    from ._models_py3 import WorkspaceCreateParameters
    from ._models_py3 import WorkspaceListResult
    from ._models_py3 import WorkspaceUpdateParameters
    from ._models_py3 import WorkspacesListByResourceGroupOptions
    from ._models_py3 import WorkspacesListOptions
except (SyntaxError, ImportError):
    from ._models import AppInsightsReference  # type: ignore
    from ._models import AutoScaleSettings  # type: ignore
    from ._models import AzureBlobFileSystemReference  # type: ignore
    from ._models import AzureFileShareReference  # type: ignore
    from ._models import AzureStorageCredentialsInfo  # type: ignore
    from ._models import BatchAIError  # type: ignore
    from ._models import CNTKsettings  # type: ignore
    from ._models import Caffe2Settings  # type: ignore
    from ._models import CaffeSettings  # type: ignore
    from ._models import ChainerSettings  # type: ignore
    from ._models import CloudErrorBody  # type: ignore
    from ._models import Cluster  # type: ignore
    from ._models import ClusterCreateParameters  # type: ignore
    from ._models import ClusterListResult  # type: ignore
    from ._models import ClusterUpdateParameters  # type: ignore
    from ._models import ClustersListByWorkspaceOptions  # type: ignore
    from ._models import ContainerSettings  # type: ignore
    from ._models import CustomMpiSettings  # type: ignore
    from ._models import CustomToolkitSettings  # type: ignore
    from ._models import DataDisks  # type: ignore
    from ._models import EnvironmentVariable  # type: ignore
    from ._models import EnvironmentVariableWithSecretValue  # type: ignore
    from ._models import Experiment  # type: ignore
    from ._models import ExperimentListResult  # type: ignore
    from ._models import ExperimentsListByWorkspaceOptions  # type: ignore
    from ._models import File  # type: ignore
    from ._models import FileListResult  # type: ignore
    from ._models import FileServer  # type: ignore
    from ._models import FileServerCreateParameters  # type: ignore
    from ._models import FileServerListResult  # type: ignore
    from ._models import FileServerReference  # type: ignore
    from ._models import FileServersListByWorkspaceOptions  # type: ignore
    from ._models import HorovodSettings  # type: ignore
    from ._models import ImageReference  # type: ignore
    from ._models import ImageSourceRegistry  # type: ignore
    from ._models import InputDirectory  # type: ignore
    from ._models import Job  # type: ignore
    from ._models import JobBasePropertiesConstraints  # type: ignore
    from ._models import JobCreateParameters  # type: ignore
    from ._models import JobListResult  # type: ignore
    from ._models import JobPreparation  # type: ignore
    from ._models import JobPropertiesConstraints  # type: ignore
    from ._models import JobPropertiesExecutionInfo  # type: ignore
    from ._models import JobsListByExperimentOptions  # type: ignore
    from ._models import JobsListOutputFilesOptions  # type: ignore
    from ._models import KeyVaultSecretReference  # type: ignore
    from ._models import ListUsagesResult  # type: ignore
    from ._models import ManualScaleSettings  # type: ignore
    from ._models import MountSettings  # type: ignore
    from ._models import MountVolumes  # type: ignore
    from ._models import NameValuePair  # type: ignore
    from ._models import NodeSetup  # type: ignore
    from ._models import NodeStateCounts  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import OutputDirectory  # type: ignore
    from ._models import PerformanceCountersSettings  # type: ignore
    from ._models import PrivateRegistryCredentials  # type: ignore
    from ._models import ProxyResource  # type: ignore
    from ._models import PyTorchSettings  # type: ignore
    from ._models import RemoteLoginInformation  # type: ignore
    from ._models import RemoteLoginInformationListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceId  # type: ignore
    from ._models import ScaleSettings  # type: ignore
    from ._models import SetupTask  # type: ignore
    from ._models import SshConfiguration  # type: ignore
    from ._models import TensorFlowSettings  # type: ignore
    from ._models import UnmanagedFileSystemReference  # type: ignore
    from ._models import Usage  # type: ignore
    from ._models import UsageName  # type: ignore
    from ._models import UserAccountSettings  # type: ignore
    from ._models import VirtualMachineConfiguration  # type: ignore
    from ._models import Workspace  # type: ignore
    from ._models import WorkspaceCreateParameters  # type: ignore
    from ._models import WorkspaceListResult  # type: ignore
    from ._models import WorkspaceUpdateParameters  # type: ignore
    from ._models import WorkspacesListByResourceGroupOptions  # type: ignore
    from ._models import WorkspacesListOptions  # type: ignore

from ._batch_ai_enums import (
    AllocationState,
    CachingType,
    DeallocationOption,
    ExecutionState,
    FileServerProvisioningState,
    FileType,
    JobPriority,
    ProvisioningState,
    StorageAccountType,
    ToolType,
    UsageUnit,
    VmPriority,
)

__all__ = [
    'AppInsightsReference',
    'AutoScaleSettings',
    'AzureBlobFileSystemReference',
    'AzureFileShareReference',
    'AzureStorageCredentialsInfo',
    'BatchAIError',
    'CNTKsettings',
    'Caffe2Settings',
    'CaffeSettings',
    'ChainerSettings',
    'CloudErrorBody',
    'Cluster',
    'ClusterCreateParameters',
    'ClusterListResult',
    'ClusterUpdateParameters',
    'ClustersListByWorkspaceOptions',
    'ContainerSettings',
    'CustomMpiSettings',
    'CustomToolkitSettings',
    'DataDisks',
    'EnvironmentVariable',
    'EnvironmentVariableWithSecretValue',
    'Experiment',
    'ExperimentListResult',
    'ExperimentsListByWorkspaceOptions',
    'File',
    'FileListResult',
    'FileServer',
    'FileServerCreateParameters',
    'FileServerListResult',
    'FileServerReference',
    'FileServersListByWorkspaceOptions',
    'HorovodSettings',
    'ImageReference',
    'ImageSourceRegistry',
    'InputDirectory',
    'Job',
    'JobBasePropertiesConstraints',
    'JobCreateParameters',
    'JobListResult',
    'JobPreparation',
    'JobPropertiesConstraints',
    'JobPropertiesExecutionInfo',
    'JobsListByExperimentOptions',
    'JobsListOutputFilesOptions',
    'KeyVaultSecretReference',
    'ListUsagesResult',
    'ManualScaleSettings',
    'MountSettings',
    'MountVolumes',
    'NameValuePair',
    'NodeSetup',
    'NodeStateCounts',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'OutputDirectory',
    'PerformanceCountersSettings',
    'PrivateRegistryCredentials',
    'ProxyResource',
    'PyTorchSettings',
    'RemoteLoginInformation',
    'RemoteLoginInformationListResult',
    'Resource',
    'ResourceId',
    'ScaleSettings',
    'SetupTask',
    'SshConfiguration',
    'TensorFlowSettings',
    'UnmanagedFileSystemReference',
    'Usage',
    'UsageName',
    'UserAccountSettings',
    'VirtualMachineConfiguration',
    'Workspace',
    'WorkspaceCreateParameters',
    'WorkspaceListResult',
    'WorkspaceUpdateParameters',
    'WorkspacesListByResourceGroupOptions',
    'WorkspacesListOptions',
    'AllocationState',
    'CachingType',
    'DeallocationOption',
    'ExecutionState',
    'FileServerProvisioningState',
    'FileType',
    'JobPriority',
    'ProvisioningState',
    'StorageAccountType',
    'ToolType',
    'UsageUnit',
    'VmPriority',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=no-self-use, line-too-long, protected-access, too-few-public-methods, unused-argument
from knack.log import get_logger

from ..aaz.latest.sig.gallery_application.version import Create as _SigGalleryApplicationVersionCreate, Update as _SigGalleryApplicationVersionUpdate

logger = get_logger(__name__)


class SigGalleryApplicationVersionCreate(_SigGalleryApplicationVersionCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.install_command._required = True
        args_schema.package_file_link._required = True
        args_schema.remove_command._required = True
        return args_schema


class SiggalleryApplicationversionUpdate(_SigGalleryApplicationVersionUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.package_file_link._required = True
        return args_schema



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AggregatedReplicationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This is the aggregated replication status based on all the regional replication status flags."""

    UNKNOWN = "Unknown"
    IN_PROGRESS = "InProgress"
    COMPLETED = "Completed"
    FAILED = "Failed"


class Architecture(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The architecture of the image. Applicable to OS disks only."""

    X64 = "x64"
    ARM64 = "Arm64"


class ConfidentialVMEncryptionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """confidential VM encryption types."""

    ENCRYPTED_VM_GUEST_STATE_ONLY_WITH_PMK = "EncryptedVMGuestStateOnlyWithPmk"
    ENCRYPTED_WITH_PMK = "EncryptedWithPmk"
    ENCRYPTED_WITH_CMK = "EncryptedWithCmk"


class ExtendedLocationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of extendedLocation."""

    EDGE_ZONE = "EdgeZone"


class GalleryExpandParams(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """GalleryExpandParams."""

    SHARING_PROFILE_GROUPS = "SharingProfile/Groups"


class GalleryExtendedLocationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """It is type of the extended location."""

    EDGE_ZONE = "EdgeZone"
    UNKNOWN = "Unknown"


class GalleryProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The provisioning state, which only appears in the response."""

    CREATING = "Creating"
    UPDATING = "Updating"
    FAILED = "Failed"
    SUCCEEDED = "Succeeded"
    DELETING = "Deleting"
    MIGRATING = "Migrating"


class GallerySharingPermissionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows you to specify the permission of sharing gallery. :code:`<br>`\\
    :code:`<br>` Possible values are: :code:`<br>`\\ :code:`<br>` **Private** :code:`<br>`\\
    :code:`<br>` **Groups** :code:`<br>`\\ :code:`<br>` **Community**.
    """

    PRIVATE = "Private"
    GROUPS = "Groups"
    COMMUNITY = "Community"


class HostCaching(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The host caching of the disk. Valid values are 'None', 'ReadOnly', and 'ReadWrite'."""

    NONE = "None"
    READ_ONLY = "ReadOnly"
    READ_WRITE = "ReadWrite"


class HyperVGeneration(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The hypervisor generation of the Virtual Machine. Applicable to OS disks only."""

    V1 = "V1"
    V2 = "V2"


class OperatingSystemStateTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows the user to specify whether the virtual machines created under this image
    are 'Generalized' or 'Specialized'.
    """

    GENERALIZED = "Generalized"
    SPECIALIZED = "Specialized"


class OperatingSystemTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows you to specify the supported type of the OS that application is built for.
    :code:`<br>`\\ :code:`<br>` Possible values are: :code:`<br>`\\ :code:`<br>` **Windows**
    :code:`<br>`\\ :code:`<br>` **Linux**.
    """

    WINDOWS = "Windows"
    LINUX = "Linux"


class ReplicationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Optional parameter which specifies the mode to be used for replication. This property is not
    updatable.
    """

    FULL = "Full"
    SHALLOW = "Shallow"


class ReplicationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This is the regional replication state."""

    UNKNOWN = "Unknown"
    REPLICATING = "Replicating"
    COMPLETED = "Completed"
    FAILED = "Failed"


class ReplicationStatusTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ReplicationStatusTypes."""

    REPLICATION_STATUS = "ReplicationStatus"


class SelectPermissions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SelectPermissions."""

    PERMISSIONS = "Permissions"


class SharedGalleryHostCaching(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The host caching of the disk. Valid values are 'None', 'ReadOnly', and 'ReadWrite'."""

    NONE = "None"
    READ_ONLY = "ReadOnly"
    READ_WRITE = "ReadWrite"


class SharedToValues(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SharedToValues."""

    TENANT = "tenant"


class SharingProfileGroupTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows you to specify the type of sharing group. :code:`<br>`\\ :code:`<br>`
    Possible values are: :code:`<br>`\\ :code:`<br>` **Subscriptions** :code:`<br>`\\ :code:`<br>`
    **AADTenants**.
    """

    SUBSCRIPTIONS = "Subscriptions"
    AAD_TENANTS = "AADTenants"


class SharingState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The sharing state of the gallery, which only appears in the response."""

    SUCCEEDED = "Succeeded"
    IN_PROGRESS = "InProgress"
    FAILED = "Failed"
    UNKNOWN = "Unknown"


class SharingUpdateOperationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows you to specify the operation type of gallery sharing update.
    :code:`<br>`\\ :code:`<br>` Possible values are: :code:`<br>`\\ :code:`<br>` **Add**
    :code:`<br>`\\ :code:`<br>` **Remove** :code:`<br>`\\ :code:`<br>` **Reset**.
    """

    ADD = "Add"
    REMOVE = "Remove"
    RESET = "Reset"
    ENABLE_COMMUNITY = "EnableCommunity"


class StorageAccountType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the storage account type to be used to store the image. This property is not
    updatable.
    """

    STANDARD_LRS = "Standard_LRS"
    STANDARD_ZRS = "Standard_ZRS"
    PREMIUM_LRS = "Premium_LRS"
    STANDARD_SSD_LRS = "StandardSSD_LRS"



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, Callable, Dict, Iterable, Optional, TypeVar, Union

from msrest import Serializer

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling

from .. import models as _models
from .._vendor import _convert_request, _format_url_section
T = TypeVar('T')
JSONType = Any
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False

def build_get_request(
    subscription_id: str,
    resource_group_name: str,
    partner_registration_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "partnerRegistrationName": _SERIALIZER.url("partner_registration_name", partner_registration_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )


def build_create_or_update_request_initial(
    subscription_id: str,
    resource_group_name: str,
    partner_registration_name: str,
    *,
    json: JSONType = None,
    content: Any = None,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "partnerRegistrationName": _SERIALIZER.url("partner_registration_name", partner_registration_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="PUT",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        json=json,
        content=content,
        **kwargs
    )


def build_delete_request_initial(
    subscription_id: str,
    resource_group_name: str,
    partner_registration_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "partnerRegistrationName": _SERIALIZER.url("partner_registration_name", partner_registration_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    return HttpRequest(
        method="DELETE",
        url=_url,
        params=_query_parameters,
        **kwargs
    )


def build_update_request_initial(
    subscription_id: str,
    resource_group_name: str,
    partner_registration_name: str,
    *,
    json: JSONType = None,
    content: Any = None,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "partnerRegistrationName": _SERIALIZER.url("partner_registration_name", partner_registration_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="PATCH",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        json=json,
        content=content,
        **kwargs
    )


def build_list_by_subscription_request(
    subscription_id: str,
    *,
    filter: Optional[str] = None,
    top: Optional[int] = None,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.EventGrid/partnerRegistrations")
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')
    if filter is not None:
        _query_parameters['$filter'] = _SERIALIZER.query("filter", filter, 'str')
    if top is not None:
        _query_parameters['$top'] = _SERIALIZER.query("top", top, 'int')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )


def build_list_by_resource_group_request(
    subscription_id: str,
    resource_group_name: str,
    *,
    filter: Optional[str] = None,
    top: Optional[int] = None,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')
    if filter is not None:
        _query_parameters['$filter'] = _SERIALIZER.query("filter", filter, 'str')
    if top is not None:
        _query_parameters['$top'] = _SERIALIZER.query("top", top, 'int')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )

class PartnerRegistrationsOperations(object):
    """PartnerRegistrationsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.eventgrid.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def get(
        self,
        resource_group_name: str,
        partner_registration_name: str,
        **kwargs: Any
    ) -> "_models.PartnerRegistration":
        """Get a partner registration.

        Gets a partner registration with the specified parameters.

        :param resource_group_name: The name of the resource group within the user's subscription.
        :type resource_group_name: str
        :param partner_registration_name: Name of the partner registration.
        :type partner_registration_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: PartnerRegistration, or the result of cls(response)
        :rtype: ~azure.mgmt.eventgrid.models.PartnerRegistration
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistration"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        
        request = build_get_request(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            partner_registration_name=partner_registration_name,
            api_version=api_version,
            template_url=self.get.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('PartnerRegistration', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore


    def _create_or_update_initial(
        self,
        resource_group_name: str,
        partner_registration_name: str,
        partner_registration_info: "_models.PartnerRegistration",
        **kwargs: Any
    ) -> "_models.PartnerRegistration":
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistration"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(partner_registration_info, 'PartnerRegistration')

        request = build_create_or_update_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            partner_registration_name=partner_registration_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            template_url=self._create_or_update_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize('PartnerRegistration', pipeline_response)

        if response.status_code == 202:
            deserialized = self._deserialize('PartnerRegistration', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    _create_or_update_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore


    @distributed_trace
    def begin_create_or_update(
        self,
        resource_group_name: str,
        partner_registration_name: str,
        partner_registration_info: "_models.PartnerRegistration",
        **kwargs: Any
    ) -> LROPoller["_models.PartnerRegistration"]:
        """Create a partner registration.

        Creates a new partner registration with the specified parameters.

        :param resource_group_name: The name of the resource group within the user's subscription.
        :type resource_group_name: str
        :param partner_registration_name: Name of the partner registration.
        :type partner_registration_name: str
        :param partner_registration_info: PartnerRegistration information.
        :type partner_registration_info: ~azure.mgmt.eventgrid.models.PartnerRegistration
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either PartnerRegistration or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.eventgrid.models.PartnerRegistration]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistration"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._create_or_update_initial(
                resource_group_name=resource_group_name,
                partner_registration_name=partner_registration_name,
                partner_registration_info=partner_registration_info,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('PartnerRegistration', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_create_or_update.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore

    def _delete_initial(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        partner_registration_name: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        
        request = build_delete_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            partner_registration_name=partner_registration_name,
            api_version=api_version,
            template_url=self._delete_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    _delete_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore


    @distributed_trace
    def begin_delete(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        partner_registration_name: str,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Delete a partner registration.

        Deletes a partner registration with the specified parameters.

        :param resource_group_name: The name of the resource group within the user's subscription.
        :type resource_group_name: str
        :param partner_registration_name: Name of the partner registration.
        :type partner_registration_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
        polling = kwargs.pop('polling', True)  # type: Union[bool, PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._delete_initial(
                resource_group_name=resource_group_name,
                partner_registration_name=partner_registration_name,
                api_version=api_version,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_delete.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore

    def _update_initial(
        self,
        resource_group_name: str,
        partner_registration_name: str,
        partner_registration_update_parameters: "_models.PartnerRegistrationUpdateParameters",
        **kwargs: Any
    ) -> Optional["_models.PartnerRegistration"]:
        cls = kwargs.pop('cls', None)  # type: ClsType[Optional["_models.PartnerRegistration"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(partner_registration_update_parameters, 'PartnerRegistrationUpdateParameters')

        request = build_update_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            partner_registration_name=partner_registration_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            template_url=self._update_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = None
        if response.status_code == 201:
            deserialized = self._deserialize('PartnerRegistration', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    _update_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore


    @distributed_trace
    def begin_update(
        self,
        resource_group_name: str,
        partner_registration_name: str,
        partner_registration_update_parameters: "_models.PartnerRegistrationUpdateParameters",
        **kwargs: Any
    ) -> LROPoller["_models.PartnerRegistration"]:
        """Update a partner registration.

        Updates a partner registration with the specified parameters.

        :param resource_group_name: The name of the resource group within the user's subscription.
        :type resource_group_name: str
        :param partner_registration_name: Name of the partner registration.
        :type partner_registration_name: str
        :param partner_registration_update_parameters: Partner registration update information.
        :type partner_registration_update_parameters:
         ~azure.mgmt.eventgrid.models.PartnerRegistrationUpdateParameters
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be ARMPolling. Pass in False for this
         operation to not poll, or pass in your own initialized polling object for a personal polling
         strategy.
        :paramtype polling: bool or ~azure.core.polling.PollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of LROPoller that returns either PartnerRegistration or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.eventgrid.models.PartnerRegistration]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, PollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistration"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = self._update_initial(
                resource_group_name=resource_group_name,
                partner_registration_name=partner_registration_name,
                partner_registration_update_parameters=partner_registration_update_parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('PartnerRegistration', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = ARMPolling(lro_delay, **kwargs)
        elif polling is False: polling_method = NoPolling()
        else: polling_method = polling
        if cont_token:
            return LROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return LROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_update.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations/{partnerRegistrationName}"}  # type: ignore

    @distributed_trace
    def list_by_subscription(
        self,
        filter: Optional[str] = None,
        top: Optional[int] = None,
        **kwargs: Any
    ) -> Iterable["_models.PartnerRegistrationsListResult"]:
        """List partner registrations under an Azure subscription.

        List all the partner registrations under an Azure subscription.

        :param filter: The query used to filter the search results using OData syntax. Filtering is
         permitted on the 'name' property only and with limited number of OData operations. These
         operations are: the 'contains' function as well as the following logical operations: not, and,
         or, eq (for equal), and ne (for not equal). No arithmetic operations are supported. The
         following is a valid filter example: $filter=contains(namE, 'PATTERN') and name ne 'PATTERN-1'.
         The following is not a valid filter example: $filter=location eq 'westus'. Default value is
         None.
        :type filter: str
        :param top: The number of results to return per page for the list operation. Valid range for
         top parameter is 1 to 100. If not specified, the default number of results to be returned is 20
         items per page. Default value is None.
        :type top: int
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either PartnerRegistrationsListResult or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.eventgrid.models.PartnerRegistrationsListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistrationsListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_by_subscription_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    filter=filter,
                    top=top,
                    template_url=self.list_by_subscription.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_by_subscription_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    filter=filter,
                    top=top,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("PartnerRegistrationsListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list_by_subscription.metadata = {'url': "/subscriptions/{subscriptionId}/providers/Microsoft.EventGrid/partnerRegistrations"}  # type: ignore

    @distributed_trace
    def list_by_resource_group(
        self,
        resource_group_name: str,
        filter: Optional[str] = None,
        top: Optional[int] = None,
        **kwargs: Any
    ) -> Iterable["_models.PartnerRegistrationsListResult"]:
        """List partner registrations under a resource group.

        List all the partner registrations under a resource group.

        :param resource_group_name: The name of the resource group within the user's subscription.
        :type resource_group_name: str
        :param filter: The query used to filter the search results using OData syntax. Filtering is
         permitted on the 'name' property only and with limited number of OData operations. These
         operations are: the 'contains' function as well as the following logical operations: not, and,
         or, eq (for equal), and ne (for not equal). No arithmetic operations are supported. The
         following is a valid filter example: $filter=contains(namE, 'PATTERN') and name ne 'PATTERN-1'.
         The following is not a valid filter example: $filter=location eq 'westus'. Default value is
         None.
        :type filter: str
        :param top: The number of results to return per page for the list operation. Valid range for
         top parameter is 1 to 100. If not specified, the default number of results to be returned is 20
         items per page. Default value is None.
        :type top: int
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either PartnerRegistrationsListResult or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.eventgrid.models.PartnerRegistrationsListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PartnerRegistrationsListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_by_resource_group_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    api_version=api_version,
                    filter=filter,
                    top=top,
                    template_url=self.list_by_resource_group.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_by_resource_group_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    api_version=api_version,
                    filter=filter,
                    top=top,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("PartnerRegistrationsListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list_by_resource_group.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/partnerRegistrations"}  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import Attributes
from ._models_py3 import BackupSecretResult
from ._models_py3 import DeletedSecretBundle
from ._models_py3 import DeletedSecretItem
from ._models_py3 import DeletedSecretListResult
from ._models_py3 import Error
from ._models_py3 import KeyVaultError
from ._models_py3 import SecretAttributes
from ._models_py3 import SecretBundle
from ._models_py3 import SecretItem
from ._models_py3 import SecretListResult
from ._models_py3 import SecretProperties
from ._models_py3 import SecretRestoreParameters
from ._models_py3 import SecretSetParameters
from ._models_py3 import SecretUpdateParameters

from ._key_vault_client_enums import DeletionRecoveryLevel
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Attributes",
    "BackupSecretResult",
    "DeletedSecretBundle",
    "DeletedSecretItem",
    "DeletedSecretListResult",
    "Error",
    "KeyVaultError",
    "SecretAttributes",
    "SecretBundle",
    "SecretItem",
    "SecretListResult",
    "SecretProperties",
    "SecretRestoreParameters",
    "SecretSetParameters",
    "SecretUpdateParameters",
    "DeletionRecoveryLevel",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AgreementTerms
    from ._models_py3 import ErrorResponse
    from ._models_py3 import ErrorResponseError
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import Resource
    from ._models_py3 import SystemData
except (SyntaxError, ImportError):
    from ._models import AgreementTerms  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import ErrorResponseError  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import SystemData  # type: ignore

from ._marketplace_ordering_agreements_enums import (
    CreatedByType,
    OfferType,
)

__all__ = [
    'AgreementTerms',
    'ErrorResponse',
    'ErrorResponseError',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'Resource',
    'SystemData',
    'CreatedByType',
    'OfferType',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long, too-many-statements

import json

from azure.cli.core.aaz import has_value, AAZIntArg, AAZBoolArg, AAZStrArg
from azure.cli.command_modules.monitor.actions import AAZCustomListArg
from azure.cli.command_modules.monitor._autoscale_util import get_autoscale_default_profile
from azure.cli.command_modules.monitor.aaz.latest.monitor.autoscale._update import Update as _AutoScaleUpdate
from azure.cli.command_modules.monitor.operations.autoscale_settings import (
    update_add_actions, update_remove_actions, _parse_action_removals,
)
from azure.cli.core.azclierror import InvalidArgumentValueError


class AutoScaleUpdate(_AutoScaleUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.notifications._registered = False
        args_schema.profiles._registered = False
        args_schema.target_resource_location._registered = False
        args_schema.target_resource_uri._registered = False
        args_schema.count = AAZIntArg(
            options=["--count"],
            help='The numer of instances to use. If used with --min/max-count, the default number of instances to use.',
            arg_group="Instance Limit",
        )
        args_schema.min_count = AAZIntArg(
            options=["--min-count"],
            help='The minimum number of instances.',
            arg_group="Instance Limit",
        )
        args_schema.max_count = AAZIntArg(
            options=["--max-count"],
            help='The maximum number of instances.',
            arg_group="Instance Limit",
        )
        args_schema.add_actions = AAZCustomListArg(
            options=["--add-actions"],
            singular_options=['--add-action', '-a'],
            help="Add an action to fire when a scaling event occurs." + '''
        Usage:   --add-action TYPE KEY [ARG ...]
        Email:   --add-action email bob@contoso.com ann@contoso.com
        Webhook: --add-action webhook https://www.contoso.com/alert apiKey=value
        Webhook: --add-action webhook https://www.contoso.com/alert?apiKey=value
        Multiple actions can be specified by using more than one `--add-action` argument.
        ''',
            arg_group="Notification",
        )
        args_schema.add_actions.Element = AAZCustomListArg()
        args_schema.add_actions.Element.Element = AAZStrArg()

        args_schema.remove_actions = AAZCustomListArg(
            options=["--remove-actions"],
            singular_options=['--remove-action', '-r'],
            help="Remove one or more actions." + '''
        Usage:   --remove-action TYPE KEY [KEY ...]
        Email:   --remove-action email bob@contoso.com ann@contoso.com
        Webhook: --remove-action webhook https://contoso.com/alert https://alerts.contoso.com.
        ''',
            arg_group="Notification",
        )
        args_schema.remove_actions.Element = AAZCustomListArg()
        args_schema.remove_actions.Element.Element = AAZStrArg()

        args_schema.email_administrator = AAZBoolArg(
            options=["--email-administrator"],
            help='Send email to subscription administrator on scaling.',
            arg_group="Notification",
        )
        args_schema.email_coadministrators = AAZBoolArg(
            options=["--email-coadministrators"],
            help='Send email to subscription co-administrators on scaling.',
            arg_group="Notification",
        )

        return args_schema

    def pre_instance_update(self, instance):
        args = self.ctx.args
        add_actions = update_add_actions(args)
        remove_actions = update_remove_actions(args)
        if has_value(args.count) or has_value(args.min_count) or has_value(args.max_count):
            default_profile = get_autoscale_default_profile(instance)
            curr_count = default_profile["capacity"]["default"]
            curr_min = default_profile["capacity"]["minimum"]
            curr_max = default_profile["capacity"]["maximum"]
            is_fixed_count = curr_count == curr_min and curr_count == curr_max

            # check for special case where count is used to indicate fixed value and only
            # count is updated
            if has_value(args.count) and is_fixed_count and not has_value(args.min_count) and not has_value(args.max_count):
                args.min_count = args.count.to_serialized_data()
                args.max_count = args.count.to_serialized_data()

            count = curr_count if not has_value(args.count) else args.count.to_serialized_data()
            min_count = curr_min if not has_value(args.min_count) else args.min_count.to_serialized_data()
            max_count = curr_max if not has_value(args.max_count) else args.max_count.to_serialized_data()

            # There may be multiple "default" profiles. All need to updated.
            for profile in instance.properties.profiles:
                if has_value(profile.fixed_date):
                    continue
                if has_value(profile.recurrence):
                    try:
                        # portal denotes the "default" pairs by using a JSON string for their name
                        # so if it can be decoded, we know this is a default profile
                        json.loads(profile.name.to_serialized_data())
                    except ValueError:
                        continue
                profile.capacity.default = str(count)
                profile.capacity.minimum = str(min_count)
                profile.capacity.maximum = str(max_count)
        updated_notification = None
        if instance.properties.notifications:
            retained_notification = []
            for x in instance.properties.notifications:
                note = x.to_serialized_data()
                if note['operation'].lower() == 'scale':
                    updated_notification = note
                else:
                    retained_notification.append(note)
            instance.properties.notifications = retained_notification
        else:
            instance.properties.notifications = []

        if updated_notification is None:
            updated_notification = {
                "operation": "scale",
                "email": {
                    "customEmails": []
                },
                "webhooks": []
            }

        # process removals
        if len(remove_actions) > 0:
            removed_emails, removed_webhooks = _parse_action_removals(remove_actions)
            updated_notification['email']['customEmails'] = \
                [x for x in updated_notification['email']['customEmails'] if x not in removed_emails]
            updated_notification['webhooks'] = \
                [x for x in updated_notification['webhooks'] if x['serviceUri'] not in removed_webhooks]

        # process additions
        for action in add_actions:
            if action["key"] == "email":
                for email in action["value"]["customEmails"]:
                    updated_notification['email']['customEmails'].append(email)
            elif action["key"] == "webhook":
                updated_notification['webhooks'].append(action["value"])
        if has_value(args.email_administrator):
            updated_notification['email']['sendToSubscriptionAdministrator'] = args.email_administrator.to_serialized_data()
        if has_value(args.email_coadministrators):
            updated_notification['email']['sendToSubscriptionCoAdministrators'] = args.email_coadministrators.to_serialized_data()

        instance.properties.notifications.append(updated_notification)

        if has_value(args.scale_look_ahead_time) and not has_value(args.scale_mode) \
                and not has_value(instance.properties.predictive_autoscale_policy):
            raise InvalidArgumentValueError('scale-mode is required for setting scale-look-ahead-time.')

    def _output(self, *args, **kwargs):
        from azure.cli.core.aaz import AAZUndefined
        # When the name field conflicts, the name in inner layer is ignored and the outer layer is applied
        if has_value(self.ctx.vars.instance.properties.name):
            self.ctx.vars.instance.properties.name = AAZUndefined
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from six import with_metaclass
from azure.core import CaseInsensitiveEnumMeta


class AccessRightsDescription(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Rights that this key has.
    """

    SERVICE_CONFIG = "ServiceConfig"
    ENROLLMENT_READ = "EnrollmentRead"
    ENROLLMENT_WRITE = "EnrollmentWrite"
    DEVICE_CONNECT = "DeviceConnect"
    REGISTRATION_STATUS_READ = "RegistrationStatusRead"
    REGISTRATION_STATUS_WRITE = "RegistrationStatusWrite"

class AllocationPolicy(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Allocation policy to be used by this provisioning service.
    """

    HASHED = "Hashed"
    GEO_LATENCY = "GeoLatency"
    STATIC = "Static"

class CertificatePurpose(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):

    CLIENT_AUTHENTICATION = "clientAuthentication"
    SERVER_AUTHENTICATION = "serverAuthentication"

class CreatedByType(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The type of identity that created the resource.
    """

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"

class IotDpsSku(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Sku name.
    """

    S1 = "S1"

class IpFilterActionType(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The desired action for requests captured by this rule.
    """

    ACCEPT = "Accept"
    REJECT = "Reject"

class IpFilterTargetType(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Target for requests captured by this rule.
    """

    ALL = "all"
    SERVICE_API = "serviceApi"
    DEVICE_API = "deviceApi"

class NameUnavailabilityReason(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """specifies the reason a name is unavailable
    """

    INVALID = "Invalid"
    ALREADY_EXISTS = "AlreadyExists"

class PrivateLinkServiceConnectionStatus(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The status of a private endpoint connection
    """

    PENDING = "Pending"
    APPROVED = "Approved"
    REJECTED = "Rejected"
    DISCONNECTED = "Disconnected"

class PublicNetworkAccess(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Whether requests from Public Network are allowed
    """

    ENABLED = "Enabled"
    DISABLED = "Disabled"

class State(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Current state of the provisioning service.
    """

    ACTIVATING = "Activating"
    ACTIVE = "Active"
    DELETING = "Deleting"
    DELETED = "Deleted"
    ACTIVATION_FAILED = "ActivationFailed"
    DELETION_FAILED = "DeletionFailed"
    TRANSITIONING = "Transitioning"
    SUSPENDING = "Suspending"
    SUSPENDED = "Suspended"
    RESUMING = "Resuming"
    FAILING_OVER = "FailingOver"
    FAILOVER_FAILED = "FailoverFailed"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum, EnumMeta
from six import with_metaclass

class _CaseInsensitiveEnumMeta(EnumMeta):
    def __getitem__(self, name):
        return super().__getitem__(name.upper())

    def __getattr__(cls, name):
        """Return the enum member matching `name`
        We use __getattr__ instead of descriptors or inserting into the enum
        class' __dict__ in order to support `name` and `value` being both
        properties for enum members (which live in the class' __dict__) and
        enum members themselves.
        """
        try:
            return cls._member_map_[name.upper()]
        except KeyError:
            raise AttributeError(name)


class ProvisioningState(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The provisioning state of the resource. This is a read-only property and any attempt to set
    this value will be ignored.
    """

    CREATING = "Creating"
    UPDATING = "Updating"
    DELETING = "Deleting"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"

class RecordType(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):

    A = "A"
    AAAA = "AAAA"
    CNAME = "CNAME"
    MX = "MX"
    PTR = "PTR"
    SOA = "SOA"
    SRV = "SRV"
    TXT = "TXT"

class VirtualNetworkLinkState(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The status of the virtual network link to the Private DNS zone. Possible values are
    'InProgress' and 'Done'. This is a read-only property and any attempt to set this value will be
    ignored.
    """

    IN_PROGRESS = "InProgress"
    COMPLETED = "Completed"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, Callable, Dict, IO, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._serialization import Serializer

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_provision_request(
    vault_name: str,
    resource_group_name: str,
    fabric_name: str,
    container_name: str,
    protected_item_name: str,
    recovery_point_id: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.RecoveryServices/vaults/{vaultName}/backupFabrics/{fabricName}/protectionContainers/{containerName}/protectedItems/{protectedItemName}/recoveryPoints/{recoveryPointId}/provisionInstantItemRecovery",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "vaultName": _SERIALIZER.url("vault_name", vault_name, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "fabricName": _SERIALIZER.url("fabric_name", fabric_name, "str"),
        "containerName": _SERIALIZER.url("container_name", container_name, "str"),
        "protectedItemName": _SERIALIZER.url("protected_item_name", protected_item_name, "str"),
        "recoveryPointId": _SERIALIZER.url("recovery_point_id", recovery_point_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_revoke_request(
    vault_name: str,
    resource_group_name: str,
    fabric_name: str,
    container_name: str,
    protected_item_name: str,
    recovery_point_id: str,
    subscription_id: str,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-02-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.RecoveryServices/vaults/{vaultName}/backupFabrics/{fabricName}/protectionContainers/{containerName}/protectedItems/{protectedItemName}/recoveryPoints/{recoveryPointId}/revokeInstantItemRecovery",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "vaultName": _SERIALIZER.url("vault_name", vault_name, "str"),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
        "fabricName": _SERIALIZER.url("fabric_name", fabric_name, "str"),
        "containerName": _SERIALIZER.url("container_name", container_name, "str"),
        "protectedItemName": _SERIALIZER.url("protected_item_name", protected_item_name, "str"),
        "recoveryPointId": _SERIALIZER.url("recovery_point_id", recovery_point_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class ItemLevelRecoveryConnectionsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.recoveryservicesbackup.activestamp.RecoveryServicesBackupClient`'s
        :attr:`item_level_recovery_connections` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def provision(
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        recovery_point_id: str,
        parameters: _models.ILRRequestResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Provisions a script which invokes an iSCSI connection to the backup data. Executing this script
        opens a file
        explorer displaying all the recoverable files and folders. This is an asynchronous operation.
        To know the status of
        provisioning, call GetProtectedItemOperationResult API.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with the backed up items. Required.
        :type fabric_name: str
        :param container_name: Container name associated with the backed up items. Required.
        :type container_name: str
        :param protected_item_name: Backed up item name whose files/folders are to be restored.
         Required.
        :type protected_item_name: str
        :param recovery_point_id: Recovery point ID which represents backed up data. iSCSI connection
         will be provisioned
         for this backed up data. Required.
        :type recovery_point_id: str
        :param parameters: resource ILR request. Required.
        :type parameters: ~azure.mgmt.recoveryservicesbackup.activestamp.models.ILRRequestResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def provision(
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        recovery_point_id: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> None:
        """Provisions a script which invokes an iSCSI connection to the backup data. Executing this script
        opens a file
        explorer displaying all the recoverable files and folders. This is an asynchronous operation.
        To know the status of
        provisioning, call GetProtectedItemOperationResult API.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with the backed up items. Required.
        :type fabric_name: str
        :param container_name: Container name associated with the backed up items. Required.
        :type container_name: str
        :param protected_item_name: Backed up item name whose files/folders are to be restored.
         Required.
        :type protected_item_name: str
        :param recovery_point_id: Recovery point ID which represents backed up data. iSCSI connection
         will be provisioned
         for this backed up data. Required.
        :type recovery_point_id: str
        :param parameters: resource ILR request. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def provision(  # pylint: disable=inconsistent-return-statements
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        recovery_point_id: str,
        parameters: Union[_models.ILRRequestResource, IO[bytes]],
        **kwargs: Any
    ) -> None:
        """Provisions a script which invokes an iSCSI connection to the backup data. Executing this script
        opens a file
        explorer displaying all the recoverable files and folders. This is an asynchronous operation.
        To know the status of
        provisioning, call GetProtectedItemOperationResult API.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with the backed up items. Required.
        :type fabric_name: str
        :param container_name: Container name associated with the backed up items. Required.
        :type container_name: str
        :param protected_item_name: Backed up item name whose files/folders are to be restored.
         Required.
        :type protected_item_name: str
        :param recovery_point_id: Recovery point ID which represents backed up data. iSCSI connection
         will be provisioned
         for this backed up data. Required.
        :type recovery_point_id: str
        :param parameters: resource ILR request. Is either a ILRRequestResource type or a IO[bytes]
         type. Required.
        :type parameters: ~azure.mgmt.recoveryservicesbackup.activestamp.models.ILRRequestResource or
         IO[bytes]
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "ILRRequestResource")

        _request = build_provision_request(
            vault_name=vault_name,
            resource_group_name=resource_group_name,
            fabric_name=fabric_name,
            container_name=container_name,
            protected_item_name=protected_item_name,
            recovery_point_id=recovery_point_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def revoke(  # pylint: disable=inconsistent-return-statements
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        recovery_point_id: str,
        **kwargs: Any
    ) -> None:
        """Revokes an iSCSI connection which can be used to download a script. Executing this script opens
        a file explorer
        displaying all recoverable files and folders. This is an asynchronous operation.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with the backed up items. Required.
        :type fabric_name: str
        :param container_name: Container name associated with the backed up items. Required.
        :type container_name: str
        :param protected_item_name: Backed up item name whose files/folders are to be restored.
         Required.
        :type protected_item_name: str
        :param recovery_point_id: Recovery point ID which represents backed up data. iSCSI connection
         will be revoked for
         this backed up data. Required.
        :type recovery_point_id: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_revoke_request(
            vault_name=vault_name,
            resource_group_name=resource_group_name,
            fabric_name=fabric_name,
            container_name=container_name,
            protected_item_name=protected_item_name,
            recovery_point_id=recovery_point_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

from abc import ABCMeta, abstractmethod


class CryptoTransform(object, metaclass=ABCMeta):
    def __init__(self, key):
        self._key = key

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._key = None

    @abstractmethod
    def transform(self, data):
        raise NotImplementedError()


class BlockCryptoTransform(CryptoTransform):
    @abstractmethod
    def block_size(self):
        raise NotImplementedError()

    @abstractmethod
    def update(self, data):
        raise NotImplementedError()

    @abstractmethod
    def finalize(self):
        raise NotImplementedError()


class AuthenticatedCryptoTransform(object, metaclass=ABCMeta):
    @abstractmethod
    def tag(self):
        raise NotImplementedError()


class SignatureTransform(object, metaclass=ABCMeta):
    @abstractmethod
    def sign(self, digest):
        raise NotImplementedError()

    @abstractmethod
    def verify(self, digest, signature):
        raise NotImplementedError()


class DigestTransform(object, metaclass=ABCMeta):
    @abstractmethod
    def update(self, data):
        raise NotImplementedError()

    @abstractmethod
    def finalize(self, data):
        raise NotImplementedError()
